@@ -1,29 +1,22 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Library : Dynamic Array Library (dal)
-// File    : dal_ref.h : Structures which refere to containers.
-//           
-// Date    : August 26, 2000.
-// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
-//
-//========================================================================
-//
 // Copyright (C) 2000-2006 Yves Renard
 //
 // This file is a part of GETFEM++
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; version 2 of the License.
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
 //
 //========================================================================
 
@@ -32,6 +25,8 @@
 #define DAL_REF_H__
 
 /** @file dal_ref.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date August 26, 2000.
  *  @brief Provide some simple pseudo-containers.
  *  
  *  WARNING : modifiying the container infirm the validity of references.
@@ -357,31 +352,34 @@ namespace dal {
     typedef tab_ref_reg_spaced_iterator_<ITER> iterator;
     
     ITER it;
-    size_type N;
+    size_type N, i;
     
-    iterator operator ++(int) { iterator tmp = *this; it += N; return tmp; }
-    iterator operator --(int) { iterator tmp = *this; it -= N; return tmp; }
-    iterator &operator ++()   { it += N; return *this; }
-    iterator &operator --()   { it -= N; return *this; }
-    iterator &operator +=(difference_type i) { it += i * N; return *this; }
-    iterator &operator -=(difference_type i) { it -= i * N; return *this; }
-    iterator operator +(difference_type i) const 
-    { iterator itt = *this; return (itt += i); }
-    iterator operator -(difference_type i) const
-    { iterator itt = *this; return (itt -= i); }
-    difference_type operator -(const iterator &i) const
-    { return (it - i.it) / N; }
+    iterator operator ++(int) { iterator tmp = *this; i++; return tmp; }
+    iterator operator --(int) { iterator tmp = *this; i--; return tmp; }
+    iterator &operator ++()   { i++; return *this; }
+    iterator &operator --()   { i--; return *this; }
+    iterator &operator +=(difference_type ii) { i+=ii; return *this; }
+    iterator &operator -=(difference_type ii) { i-=ii; return *this; }
+    iterator operator +(difference_type ii) const 
+    { iterator itt = *this; return (itt += ii); }
+    iterator operator -(difference_type ii) const
+    { iterator itt = *this; return (itt -= ii); }
+    difference_type operator -(const iterator &ii) const
+    { return (N ? (it - ii.it) / N : 0) + i - ii.i; }
 
-    reference operator *() const { return *it; }
-    reference operator [](int ii) const { return *(it + ii * N); }
+    reference operator *() const { return *(it + i*N); }
+    reference operator [](int ii) const { return *(it + (i+ii)*N); }
 
-    bool operator ==(const iterator &i) const { return (it == i.it); }
-    bool operator !=(const iterator &i) const { return !(i == *this); }
-    bool operator < (const iterator &i) const { return (it < i.it); }
+    bool operator ==(const iterator &ii) const
+    { return (*this - ii) == difference_type(0); }
+    bool operator !=(const iterator &ii) const
+    { return  (*this - ii) != difference_type(0); }
+    bool operator < (const iterator &ii) const
+    { return (*this - ii) < difference_type(0); }
 
     tab_ref_reg_spaced_iterator_(void) {}
-    tab_ref_reg_spaced_iterator_(const ITER &iter, size_type n)
-      : it(iter), N(n) { }
+    tab_ref_reg_spaced_iterator_(const ITER &iter, size_type n, size_type ii)
+      : it(iter), N(n), i(ii) { }
     
   };
 
@@ -397,65 +395,63 @@ namespace dal {
   /**
      provide a "strided" view a of container
   */
-  template<typename ITER> class tab_ref_reg_spaced
-  {
-    public :
+  template<typename ITER> class tab_ref_reg_spaced {
+  public :
 
-      typedef typename std::iterator_traits<ITER>::value_type value_type;
-      typedef typename std::iterator_traits<ITER>::pointer    pointer;
-      typedef typename std::iterator_traits<ITER>::pointer    const_pointer;
-      typedef typename std::iterator_traits<ITER>::reference  reference;
-      typedef typename std::iterator_traits<ITER>::reference  const_reference;
-      typedef typename std::iterator_traits<ITER>::difference_type
-	                                                       difference_type;
-      typedef size_t size_type;
-      typedef tab_ref_reg_spaced_iterator_<ITER> iterator;
-      typedef iterator                          const_iterator;
-      typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
-      typedef std::reverse_iterator<iterator> reverse_iterator;
+    typedef typename std::iterator_traits<ITER>::value_type value_type;
+    typedef typename std::iterator_traits<ITER>::pointer    pointer;
+    typedef typename std::iterator_traits<ITER>::pointer    const_pointer;
+    typedef typename std::iterator_traits<ITER>::reference  reference;
+    typedef typename std::iterator_traits<ITER>::reference  const_reference;
+    typedef typename std::iterator_traits<ITER>::difference_type
+            difference_type;
+    typedef size_t size_type;
+    typedef tab_ref_reg_spaced_iterator_<ITER> iterator;
+    typedef iterator                          const_iterator;
+    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+    typedef std::reverse_iterator<iterator> reverse_iterator;
     
-    protected :
-
-      ITER begin_, end_;
-      size_type N;
-
-    public :
-
-      bool empty(void) const { return begin_ == end_; }
-      size_type size(void) const { return (end_ - begin_) / N; }
-
-      iterator begin(void) { return iterator(begin_, N); }
-      const_iterator begin(void) const { return iterator(begin_, N); }
-      iterator end(void) { return iterator(end_, N); }
-      const_iterator end(void) const { return iterator(end_, N); }
-      reverse_iterator rbegin(void) { return reverse_iterator(end()); }
-      const_reverse_iterator rbegin(void) const
-      { return const_reverse_iterator(end()); }
-      reverse_iterator rend(void) { return reverse_iterator(begin()); }
-      const_reverse_iterator rend(void) const
-      { return const_reverse_iterator(begin()); }
-
-      reference front(void) { return *begin_; }
-      const_reference front(void) const { return *begin_; }
-      reference back(void) { return *(--(end())); }
-      const_reference back(void) const { return *(--(end())); }
-      void pop_front(void) { begin_ += N; }
-
-      tab_ref_reg_spaced(void) {}
-      tab_ref_reg_spaced(const ITER &b, const ITER &e, size_type n)
-	: begin_(b), end_(e), N(n) {}
-
-
-      const_reference operator [](size_type ii) const
-      { return *(begin_ + ii * N);}
-      reference operator [](size_type ii) { return *(begin_ + ii * N); }
+  protected :
 
+    ITER begin_;
+    size_type N, size_;
+    
+  public :
+    
+    bool empty(void) const { return size_ == 0; }
+    size_type size(void) const { return size_; }
+    
+    iterator begin(void) { return iterator(begin_, N, 0); }
+    const_iterator begin(void) const { return iterator(begin_, N, 0); }
+    iterator end(void) { return iterator(begin_, N, size_); }
+    const_iterator end(void) const { return iterator(begin_, N, size_); }
+    reverse_iterator rbegin(void) { return reverse_iterator(end()); }
+    const_reverse_iterator rbegin(void) const
+    { return const_reverse_iterator(end()); }
+    reverse_iterator rend(void) { return reverse_iterator(begin()); }
+    const_reverse_iterator rend(void) const
+    { return const_reverse_iterator(begin()); }
+    
+    reference front(void) { return *begin_; }
+    const_reference front(void) const { return *begin_; }
+    reference back(void) { return *(begin_ + N * (size_-1)); }
+    const_reference back(void) const { *(begin_ + N * (size_-1)); }
+    void pop_front(void) { begin_ += N; }
+    
+    tab_ref_reg_spaced(void) {}
+    tab_ref_reg_spaced(const ITER &b, size_type n, size_type s)
+      : begin_(b), N(n), size_(s) {}
+    
+    
+    const_reference operator [](size_type ii) const
+    { return *(begin_ + ii * N);}
+    reference operator [](size_type ii) { return *(begin_ + ii * N); }
+    
   };
 
   /// iterator over a tab_ref_with_selection
   template<typename ITER, typename COND> 
-    struct tab_ref_with_selection_iterator_ : public ITER
-  {
+  struct tab_ref_with_selection_iterator_ : public ITER {
     typedef typename std::iterator_traits<ITER>::value_type value_type;
     typedef typename std::iterator_traits<ITER>::pointer    pointer;
     typedef typename std::iterator_traits<ITER>::reference  reference;
@@ -488,8 +484,6 @@ namespace dal {
     ITER begin_, end_;
     COND cond;
     
-    
-    
   public :
     
     typedef typename std::iterator_traits<ITER>::value_type value_type;
