@@ -1,34 +1,28 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Library : Generic Matrix Methods  (gmm)
-// File    : gmm_dense_qr.h : QR decomposition and QR algorithms for
-//           dense matrices.
-// Date    : September 12, 2003.
-// Authors : Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr
-//           Yves Renard <Yves.Renard@insa-toulouse.fr>
-//
-//========================================================================
-//
-// Copyright (C) 2003-2005 Yves Renard
+// Copyright (C) 2003-2006 Yves Renard
 //
 // This file is a part of GETFEM++
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; version 2 of the License.
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
 //
 //========================================================================
 
 /**@file gmm_dense_qr.h
+   @author  Caroline Lecalvez, Caroline.Lecalvez@gmm.insa-tlse.fr, Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date September 12, 2003.
    @brief Dense QR factorization.
 */
 #ifndef GMM_DENSE_QR_H
@@ -49,7 +43,7 @@ namespace gmm {
 
     size_type m = mat_nrows(A), n = mat_ncols(A);
     if (m < n) DAL_THROW(dimension_error, "dimensions mismatch");
-    
+
     std::vector<value_type> W(m), V(m);
 
     for (size_type j = 0; j < n; ++j) {
