@@ -1,29 +1,22 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Library : Generic Matrix Methods  (gmm)
-// File    : gmm_iter.h : iterations.
-//           
-// Date    : February 10, 2003.
-// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
-//
-//========================================================================
-//
-// Copyright (C) 2002-2005 Yves Renard
+// Copyright (C) 2002-2006 Yves Renard
 //
 // This file is a part of GETFEM++
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; version 2 of the License.
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
 //
 //========================================================================
 //=======================================================================
@@ -52,6 +45,8 @@
 //=======================================================================
 
 /**@file gmm_iter.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date February 10, 2003.
    @brief Iteration object.
 */
 
@@ -105,6 +100,8 @@ namespace gmm {
     double get_resmax(void) const { return resmax; }
     void set_resmax(double r) { resmax = r; }
 
+    double get_res() const { return res; }
+
     /* change the user-definable callback, called after each iteration */
     void set_callback(void (*t)(const gmm::iteration&)) {
       callback = t;
