@@ -1,33 +1,28 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Library : Generic Matrix Methods  (gmm)
-// File    : gmm_matrix.h : matrices.
-//           
-// Date    : October 13, 2002.
-// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
-//
-//========================================================================
-//
-// Copyright (C) 2002-2005 Yves Renard
+// Copyright (C) 2002-2006 Yves Renard
 //
 // This file is a part of GETFEM++
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; version 2 of the License.
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
 //
 //========================================================================
 
 /** @file gmm_matrix.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date October 13, 2002.
     @brief Declaration of some matrix types (gmm::dense_matrix,
     gmm::row_matrix, gmm::col_matrix, gmm::csc_matrix, etc.)
 */
@@ -100,7 +95,8 @@ namespace gmm
   void transposed_right_mult(const identity_matrix&,const V1 &v1,const V2 &v2) 
   { copy(v1, v2); }
   template <typename M> void copy_ident(const identity_matrix&, M &m) {
-    size_type i = 0, n = std::min(mat_nrows(m), mat_ncols(m)); clear(m);
+    size_type i = 0, n = std::min(mat_nrows(m), mat_ncols(m));
+    clear(m);
     for (; i < n; ++i) m(i,i) = typename linalg_traits<M>::value_type(1);
   }
   template <typename M> inline void copy(const identity_matrix&, M &m)
@@ -395,10 +391,10 @@ namespace gmm
     nbl = m; nbc = n;
   }
   
-  template<typename T> void dense_matrix<T>::fill(T a, T b) { 
+  template<typename T> void dense_matrix<T>::fill(T a, T b) {
     std::fill(this->begin(), this->end(), b);
-    iterator p = this->begin(), e = this->end();
-    while (p < e) { *p = a; p += nbl+1; }
+    size_type n = std::min(nbl, nbc);
+    if (a != b) for (size_type i = 0; i < n; ++i) (*this)(i,i) = a; 
   }
 
   template<typename T>  void dense_matrix<T>::out_of_range_error(void) const
@@ -436,36 +432,30 @@ namespace gmm
     typedef linalg_true index_sorted;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
-    static const_sub_row_type row(const const_row_iterator &it) {
-      return const_sub_row_type(it.it, it.it + it.ncols * it.nrows,
-				it.nrows, it.origin); 
-    }
+    static const_sub_row_type row(const const_row_iterator &it)
+    { return const_sub_row_type(*it, it.nrows, it.ncols, it.origin); }
     static const_sub_col_type col(const const_col_iterator &it)
-    { return const_sub_col_type(it.it, it.it + it.nrows, it.origin); }
-    static sub_row_type row(const row_iterator &it) {
-      return sub_row_type(it.it, it.it + it.ncols * it.nrows,
-			  it.nrows, it.origin);
-    }
+    { return const_sub_col_type(*it, *it + it.nrows, it.origin); }
+    static sub_row_type row(const row_iterator &it)
+    { return sub_row_type(*it, it.nrows, it.ncols, it.origin); }
     static sub_col_type col(const col_iterator &it)
-    { return sub_col_type(it.it, it.it + it.nrows, it.origin); }
+    { return sub_col_type(*it, *it + it.nrows, it.origin); }
     static row_iterator row_begin(this_type &m)
-    { return row_iterator(m.begin(), 1, m.nrows(), m.ncols(), &m); }
+    { return row_iterator(m.begin(), m.size() ? 1 : 0, m.nrows(), m.ncols(), 0, &m); }
     static row_iterator row_end(this_type &m)
-    { return row_iterator(m.begin()+m.nrows(), 1, m.nrows(), m.ncols(), &m); }
+    { return row_iterator(m.begin(), m.size() ? 1 : 0, m.nrows(), m.ncols(), m.nrows(), &m); }
     static const_row_iterator row_begin(const this_type &m)
-    { return const_row_iterator(m.begin(), 1, m.nrows(), m.ncols(), &m); }
-    static const_row_iterator row_end(const this_type &m) {
-      return const_row_iterator(m.begin()+m.nrows(), 1, m.nrows(),
-				m.ncols(), &m);
-    }
+    { return const_row_iterator(m.begin(), m.size() ? 1 : 0, m.nrows(), m.ncols(), 0, &m); }
+    static const_row_iterator row_end(const this_type &m)
+    { return const_row_iterator(m.begin(),  m.size() ? 1 : 0, m.nrows(), m.ncols(), m.nrows(), &m); }
     static col_iterator col_begin(this_type &m)
-    { return col_iterator(m.begin(), m.nrows(), m.nrows(), m.ncols(), &m); }
+    { return col_iterator(m.begin(), m.nrows(), m.nrows(), m.ncols(), 0, &m); }
     static col_iterator col_end(this_type &m)
-    { return col_iterator(m.end(), m.nrows(), m.nrows(), m.ncols(), &m); }
+    { return col_iterator(m.begin(), m.nrows(), m.nrows(), m.ncols(), m.ncols(), &m); }
     static const_col_iterator col_begin(const this_type &m)
-    { return const_col_iterator(m.begin(),m.nrows(),m.nrows(),m.ncols(),&m); }
+    { return const_col_iterator(m.begin(), m.nrows(), m.nrows(), m.ncols(), 0, &m); }
     static const_col_iterator col_end(const this_type &m)
-    { return const_col_iterator(m.end(), m.nrows(),m.nrows(),m.ncols(), &m); }
+    { return const_col_iterator(m.begin(),m.nrows(),m.nrows(),m.ncols(),m.ncols(), &m); }
     static origin_type* origin(this_type &m) { return &m; }
     static const origin_type* origin(const this_type &m) { return &m; }
     static void do_clear(this_type &m) { m.fill(value_type(0)); }
