@@ -1,32 +1,27 @@
 // -*- c++ -*- (enables emacs c++ mode)
 //========================================================================
 //
-// Library : Generic Matrix Methods  (gmm)
-// File    : gmm_vector.h : vectors.
-//           
-// Date    : October 13, 2002.
-// Author  : Yves Renard <Yves.Renard@insa-toulouse.fr>
-//
-//========================================================================
-//
-// Copyright (C) 2002-2005 Yves Renard
+// Copyright (C) 2002-2006 Yves Renard
 //
 // This file is a part of GETFEM++
 //
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; version 2 of the License.
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software Foundation,
-// Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
 //
 //========================================================================
 /**@file gmm_vector.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date October 13, 2002.
    @brief Declaration of the vector types (gmm::rsvector , gmm::wsvector , gmm::slvector ,..)
 */
 #ifndef GMM_VECTOR_H__
@@ -212,7 +207,7 @@ namespace gmm {
 #ifdef GMM_VERIFY
       if (c >= nbl) out_of_range_error();
 #endif
-      const_iterator it = lower_bound(c);
+      const_iterator it = this->lower_bound(c);
       if (it != this->end() && c == it->first) return it->second;
       else return T(0);
     }
@@ -473,10 +468,13 @@ namespace gmm {
 	       for (; it != ite && it->c <= j; ++it, ++iti) *iti = *it;
 	       *iti = a;
 	       break;
-      case 2 : a = *itj; a.c = i; it = itj; --it; ite = this->begin();
-	       for (; it >= ite && it->c >= i; --it, --itj) *itj = *it;
-	       *itj = a;
-	       break;
+      case 2 : a = *itj; a.c = i; it = itj; ite = this->begin();
+	if (it != ite) {
+	  --it;
+	  while (it->c >= i) { *itj = *it;  --itj; if (it==ite) break; --it; }
+	}
+	*itj = a;
+	break;
       case 3 : std::swap(iti->e, itj->e);
 	       break;
       }
